@@ -4,6 +4,7 @@
 
 extern crate alloc;
 
+use crate::cards::binary_card::{BinaryCard, BC64};
 use crate::parse::get_rank_and_suit;
 use strum::EnumIter;
 
@@ -57,63 +58,63 @@ impl CardNumber {
     //endregion
 
     //region cardnumbers
-    pub const ACE_SPADES: u32 = 268_471_337;
-    pub const KING_SPADES: u32 = 134_253_349;
-    pub const QUEEN_SPADES: u32 = 67_144_223;
-    pub const JACK_SPADES: u32 = 33_589_533;
-    pub const TEN_SPADES: u32 = 16_812_055;
-    pub const NINE_SPADES: u32 = 8_423_187;
-    pub const EIGHT_SPADES: u32 = 4_228_625;
-    pub const SEVEN_SPADES: u32 = 2_131_213;
-    pub const SIX_SPADES: u32 = 1_082_379;
-    pub const FIVE_SPADES: u32 = 557_831;
-    pub const FOUR_SPADES: u32 = 295_429;
-    pub const TREY_SPADES: u32 = 164_099;
-    pub const DEUCE_SPADES: u32 = 98_306;
-    pub const ACE_HEARTS: u32 = 268_454_953;
-    pub const KING_HEARTS: u32 = 134_236_965;
-    pub const QUEEN_HEARTS: u32 = 67_127_839;
-    pub const JACK_HEARTS: u32 = 33_573_149;
-    pub const TEN_HEARTS: u32 = 16_795_671;
-    pub const NINE_HEARTS: u32 = 8_406_803;
-    pub const EIGHT_HEARTS: u32 = 4_212_241;
-    pub const SEVEN_HEARTS: u32 = 2_114_829;
-    pub const SIX_HEARTS: u32 = 1_065_995;
-    pub const FIVE_HEARTS: u32 = 541_447;
-    pub const FOUR_HEARTS: u32 = 279_045;
-    pub const TREY_HEARTS: u32 = 147_715;
-    pub const DEUCE_HEARTS: u32 = 81_922;
-    pub const ACE_DIAMONDS: u32 = 268_446_761;
-    pub const KING_DIAMONDS: u32 = 134_228_773;
-    pub const QUEEN_DIAMONDS: u32 = 67_119_647;
-    pub const JACK_DIAMONDS: u32 = 33_564_957;
-    pub const TEN_DIAMONDS: u32 = 16_787_479;
-    pub const NINE_DIAMONDS: u32 = 8_398_611;
-    pub const EIGHT_DIAMONDS: u32 = 4_204_049;
-    pub const SEVEN_DIAMONDS: u32 = 2_106_637;
-    pub const SIX_DIAMONDS: u32 = 1_057_803;
-    pub const FIVE_DIAMONDS: u32 = 533_255;
-    pub const FOUR_DIAMONDS: u32 = 270_853;
-    pub const TREY_DIAMONDS: u32 = 139_523;
-    pub const DEUCE_DIAMONDS: u32 = 73_730;
-    pub const ACE_CLUBS: u32 = 268_442_665;
-    pub const KING_CLUBS: u32 = 134_224_677;
-    pub const QUEEN_CLUBS: u32 = 67_115_551;
-    pub const JACK_CLUBS: u32 = 33_560_861;
-    pub const TEN_CLUBS: u32 = 16_783_383;
-    pub const NINE_CLUBS: u32 = 8_394_515;
-    pub const EIGHT_CLUBS: u32 = 4_199_953;
-    pub const SEVEN_CLUBS: u32 = 2_102_541;
-    pub const SIX_CLUBS: u32 = 1_053_707;
-    pub const FIVE_CLUBS: u32 = 529_159;
-    pub const FOUR_CLUBS: u32 = 266_757;
-    pub const TREY_CLUBS: u32 = 135_427;
-    pub const DEUCE_CLUBS: u32 = 69_634;
-    pub const BLANK: u32 = 0;
+    pub const ACE_SPADES: CKCNumber = 268_471_337;
+    pub const KING_SPADES: CKCNumber = 134_253_349;
+    pub const QUEEN_SPADES: CKCNumber = 67_144_223;
+    pub const JACK_SPADES: CKCNumber = 33_589_533;
+    pub const TEN_SPADES: CKCNumber = 16_812_055;
+    pub const NINE_SPADES: CKCNumber = 8_423_187;
+    pub const EIGHT_SPADES: CKCNumber = 4_228_625;
+    pub const SEVEN_SPADES: CKCNumber = 2_131_213;
+    pub const SIX_SPADES: CKCNumber = 1_082_379;
+    pub const FIVE_SPADES: CKCNumber = 557_831;
+    pub const FOUR_SPADES: CKCNumber = 295_429;
+    pub const TREY_SPADES: CKCNumber = 164_099;
+    pub const DEUCE_SPADES: CKCNumber = 98_306;
+    pub const ACE_HEARTS: CKCNumber = 268_454_953;
+    pub const KING_HEARTS: CKCNumber = 134_236_965;
+    pub const QUEEN_HEARTS: CKCNumber = 67_127_839;
+    pub const JACK_HEARTS: CKCNumber = 33_573_149;
+    pub const TEN_HEARTS: CKCNumber = 16_795_671;
+    pub const NINE_HEARTS: CKCNumber = 8_406_803;
+    pub const EIGHT_HEARTS: CKCNumber = 4_212_241;
+    pub const SEVEN_HEARTS: CKCNumber = 2_114_829;
+    pub const SIX_HEARTS: CKCNumber = 1_065_995;
+    pub const FIVE_HEARTS: CKCNumber = 541_447;
+    pub const FOUR_HEARTS: CKCNumber = 279_045;
+    pub const TREY_HEARTS: CKCNumber = 147_715;
+    pub const DEUCE_HEARTS: CKCNumber = 81_922;
+    pub const ACE_DIAMONDS: CKCNumber = 268_446_761;
+    pub const KING_DIAMONDS: CKCNumber = 134_228_773;
+    pub const QUEEN_DIAMONDS: CKCNumber = 67_119_647;
+    pub const JACK_DIAMONDS: CKCNumber = 33_564_957;
+    pub const TEN_DIAMONDS: CKCNumber = 16_787_479;
+    pub const NINE_DIAMONDS: CKCNumber = 8_398_611;
+    pub const EIGHT_DIAMONDS: CKCNumber = 4_204_049;
+    pub const SEVEN_DIAMONDS: CKCNumber = 2_106_637;
+    pub const SIX_DIAMONDS: CKCNumber = 1_057_803;
+    pub const FIVE_DIAMONDS: CKCNumber = 533_255;
+    pub const FOUR_DIAMONDS: CKCNumber = 270_853;
+    pub const TREY_DIAMONDS: CKCNumber = 139_523;
+    pub const DEUCE_DIAMONDS: CKCNumber = 73_730;
+    pub const ACE_CLUBS: CKCNumber = 268_442_665;
+    pub const KING_CLUBS: CKCNumber = 134_224_677;
+    pub const QUEEN_CLUBS: CKCNumber = 67_115_551;
+    pub const JACK_CLUBS: CKCNumber = 33_560_861;
+    pub const TEN_CLUBS: CKCNumber = 16_783_383;
+    pub const NINE_CLUBS: CKCNumber = 8_394_515;
+    pub const EIGHT_CLUBS: CKCNumber = 4_199_953;
+    pub const SEVEN_CLUBS: CKCNumber = 2_102_541;
+    pub const SIX_CLUBS: CKCNumber = 1_053_707;
+    pub const FIVE_CLUBS: CKCNumber = 529_159;
+    pub const FOUR_CLUBS: CKCNumber = 266_757;
+    pub const TREY_CLUBS: CKCNumber = 135_427;
+    pub const DEUCE_CLUBS: CKCNumber = 69_634;
+    pub const BLANK: CKCNumber = 0;
     //endregion
 
     #[must_use]
-    pub fn filter(number: u32) -> CKCNumber {
+    pub fn filter(number: CKCNumber) -> CKCNumber {
         <CKCNumber as PokerCard>::filter(number)
     }
 }
@@ -125,10 +126,7 @@ mod card_number_tests {
     #[test]
     fn filter() {
         assert_eq!(CardNumber::filter(2), CardNumber::BLANK);
-        assert_eq!(
-            CardNumber::filter(CardNumber::NINE_CLUBS),
-            CardNumber::NINE_CLUBS
-        );
+        assert_eq!(CardNumber::filter(CardNumber::NINE_CLUBS), CardNumber::NINE_CLUBS);
     }
 }
 
@@ -298,15 +296,19 @@ mod card_suit_tests {
 
     #[rstest]
     #[case('♠', CardSuit::SPADES)]
+    #[case('♤', CardSuit::SPADES)]
     #[case('S', CardSuit::SPADES)]
     #[case('s', CardSuit::SPADES)]
     #[case('♥', CardSuit::HEARTS)]
+    #[case('♡', CardSuit::HEARTS)]
     #[case('H', CardSuit::HEARTS)]
     #[case('h', CardSuit::HEARTS)]
     #[case('♦', CardSuit::DIAMONDS)]
+    #[case('♢', CardSuit::DIAMONDS)]
     #[case('D', CardSuit::DIAMONDS)]
     #[case('d', CardSuit::DIAMONDS)]
     #[case('♣', CardSuit::CLUBS)]
+    #[case('♧', CardSuit::CLUBS)]
     #[case('C', CardSuit::CLUBS)]
     #[case('c', CardSuit::CLUBS)]
     #[case(' ', CardSuit::BLANK)]
@@ -317,6 +319,8 @@ mod card_suit_tests {
 }
 
 pub mod evaluate {
+    use crate::cards::five::Five;
+    use crate::cards::HandRanker;
     use crate::hand_rank::HandRankValue;
     use crate::{CKCNumber, CardNumber};
 
@@ -325,102 +329,27 @@ pub mod evaluate {
     #[must_use]
     #[allow(clippy::cast_possible_truncation)]
     pub fn five_cards(five_cards: [CKCNumber; 5]) -> HandRankValue {
-        if is_corrupt(five_cards) || has_dupes(five_cards) {
-            return CardNumber::BLANK as HandRankValue;
-        }
-        let i = or_rank_bits(five_cards);
-
-        if is_flush(five_cards) {
-            return crate::lookups::FLUSHES[i];
-        }
-
-        // Continue to evaluate if it's not a flush and the cards aren't
-        // unique (straight or high card).
-        let unique = unique(i);
-        match unique {
-            0 => not_unique(five_cards),
-            _ => unique,
-        }
+        Five::from(five_cards).hand_rank_value_validated()
     }
 
     #[must_use]
+    #[deprecated(since = "0.1.9", note = "use Five.is_flush()")]
     pub fn is_flush(five_cards: [CKCNumber; 5]) -> bool {
-        (five_cards[0]
-            & five_cards[1]
-            & five_cards[2]
-            & five_cards[3]
-            & five_cards[4]
-            & CardNumber::SUIT_FILTER)
-            != 0
+        (five_cards[0] & five_cards[1] & five_cards[2] & five_cards[3] & five_cards[4] & CardNumber::SUIT_FILTER) != 0
     }
 
     /// Returns a value that is made up of performing an or operation on all of the
     /// rank bit flags of the `PokerCard`.
     #[must_use]
+    #[deprecated(since = "0.1.9", note = "use Five.or_rank_bits()")]
     pub fn or_rank_bits(five_cards: [CKCNumber; 5]) -> usize {
-        ((five_cards[0] | five_cards[1] | five_cards[2] | five_cards[3] | five_cards[4]) as usize)
-            >> 16
-    }
-
-    #[allow(clippy::comparison_chain)]
-    fn find_in_products(key: usize) -> usize {
-        let mut low = 0;
-        let mut high = 4887;
-        let mut mid;
-
-        while low <= high {
-            mid = (high + low) >> 1; // divide by two
-
-            let product = crate::lookups::PRODUCTS[mid] as usize;
-            if key < product {
-                high = mid - 1;
-            } else if key > product {
-                low = mid + 1;
-            } else {
-                return mid;
-            }
-        }
-        0
-    }
-
-    fn has_dupes(c: [CKCNumber; 5]) -> bool {
-        (1..5).any(|i| c[i..].contains(&c[i - 1]))
-    }
-
-    fn is_corrupt(c: [CKCNumber; 5]) -> bool {
-        for x in &c {
-            if CardNumber::filter(*x) == CardNumber::BLANK {
-                return true;
-            }
-        }
-        false
-    }
-
-    fn multiply_primes(five_cards: [CKCNumber; 5]) -> usize {
-        ((five_cards[0] & 0xff)
-            * (five_cards[1] & 0xff)
-            * (five_cards[2] & 0xff)
-            * (five_cards[3] & 0xff)
-            * (five_cards[4] & 0xff)) as usize
-    }
-
-    fn not_unique(five_cards: [CKCNumber; 5]) -> HandRankValue {
-        crate::lookups::VALUES[find_in_products(multiply_primes(five_cards))]
-    }
-
-    #[allow(clippy::cast_possible_truncation)]
-    fn unique(index: usize) -> HandRankValue {
-        if index > POSSIBLE_COMBINATIONS {
-            return CardNumber::BLANK as HandRankValue;
-        }
-        crate::lookups::UNIQUE_5[index]
+        Five::from(five_cards).or_rank_bits() as usize
     }
 }
 
 #[cfg(test)]
 mod evaluate_tests {
     use super::*;
-    use alloc::format;
 
     #[test]
     fn five_cards_royal_flush() {
@@ -486,21 +415,6 @@ mod evaluate_tests {
         assert_eq!(evaluate::five_cards(second), 6684);
     }
 
-    #[test]
-    fn or_rank_bits() {
-        let cards = [
-            CardNumber::ACE_SPADES,
-            CardNumber::KING_SPADES,
-            CardNumber::QUEEN_SPADES,
-            CardNumber::JACK_SPADES,
-            CardNumber::TEN_SPADES,
-        ];
-
-        let or = evaluate::or_rank_bits(cards);
-        assert_eq!("0001111100000000", format!("{:016b}", or));
-        assert_eq!(or, 7936);
-    }
-
     #[test]
     fn check_dupes() {
         let hand = [
@@ -539,6 +453,7 @@ pub enum HandError {
     BlankCard,
     DuplicateCard,
     Incomplete,
+    InvalidBinaryFormat,
     InvalidCard,
     InvalidCardCount,
     InvalidIndex,
@@ -614,6 +529,65 @@ pub trait PokerCard {
         }
     }
 
+    #[must_use]
+    fn from_binary_card(bc: BinaryCard) -> CKCNumber {
+        match bc {
+            BinaryCard::ACE_SPADES => CardNumber::ACE_SPADES,
+            BinaryCard::KING_SPADES => CardNumber::KING_SPADES,
+            BinaryCard::QUEEN_SPADES => CardNumber::QUEEN_SPADES,
+            BinaryCard::JACK_SPADES => CardNumber::JACK_SPADES,
+            BinaryCard::TEN_SPADES => CardNumber::TEN_SPADES,
+            BinaryCard::NINE_SPADES => CardNumber::NINE_SPADES,
+            BinaryCard::EIGHT_SPADES => CardNumber::EIGHT_SPADES,
+            BinaryCard::SEVEN_SPADES => CardNumber::SEVEN_SPADES,
+            BinaryCard::SIX_SPADES => CardNumber::SIX_SPADES,
+            BinaryCard::FIVE_SPADES => CardNumber::FIVE_SPADES,
+            BinaryCard::FOUR_SPADES => CardNumber::FOUR_SPADES,
+            BinaryCard::TREY_SPADES => CardNumber::TREY_SPADES,
+            BinaryCard::DEUCE_SPADES => CardNumber::DEUCE_SPADES,
+            BinaryCard::ACE_HEARTS => CardNumber::ACE_HEARTS,
+            BinaryCard::KING_HEARTS => CardNumber::KING_HEARTS,
+            BinaryCard::QUEEN_HEARTS => CardNumber::QUEEN_HEARTS,
+            BinaryCard::JACK_HEARTS => CardNumber::JACK_HEARTS,
+            BinaryCard::TEN_HEARTS => CardNumber::TEN_HEARTS,
+            BinaryCard::NINE_HEARTS => CardNumber::NINE_HEARTS,
+            BinaryCard::EIGHT_HEARTS => CardNumber::EIGHT_HEARTS,
+            BinaryCard::SEVEN_HEARTS => CardNumber::SEVEN_HEARTS,
+            BinaryCard::SIX_HEARTS => CardNumber::SIX_HEARTS,
+            BinaryCard::FIVE_HEARTS => CardNumber::FIVE_HEARTS,
+            BinaryCard::FOUR_HEARTS => CardNumber::FOUR_HEARTS,
+            BinaryCard::TREY_HEARTS => CardNumber::TREY_HEARTS,
+            BinaryCard::DEUCE_HEARTS => CardNumber::DEUCE_HEARTS,
+            BinaryCard::ACE_DIAMONDS => CardNumber::ACE_DIAMONDS,
+            BinaryCard::KING_DIAMONDS => CardNumber::KING_DIAMONDS,
+            BinaryCard::QUEEN_DIAMONDS => CardNumber::QUEEN_DIAMONDS,
+            BinaryCard::JACK_DIAMONDS => CardNumber::JACK_DIAMONDS,
+            BinaryCard::TEN_DIAMONDS => CardNumber::TEN_DIAMONDS,
+            BinaryCard::NINE_DIAMONDS => CardNumber::NINE_DIAMONDS,
+            BinaryCard::EIGHT_DIAMONDS => CardNumber::EIGHT_DIAMONDS,
+            BinaryCard::SEVEN_DIAMONDS => CardNumber::SEVEN_DIAMONDS,
+            BinaryCard::SIX_DIAMONDS => CardNumber::SIX_DIAMONDS,
+            BinaryCard::FIVE_DIAMONDS => CardNumber::FIVE_DIAMONDS,
+            BinaryCard::FOUR_DIAMONDS => CardNumber::FOUR_DIAMONDS,
+            BinaryCard::TREY_DIAMONDS => CardNumber::TREY_DIAMONDS,
+            BinaryCard::DEUCE_DIAMONDS => CardNumber::DEUCE_DIAMONDS,
+            BinaryCard::ACE_CLUBS => CardNumber::ACE_CLUBS,
+            BinaryCard::KING_CLUBS => CardNumber::KING_CLUBS,
+            BinaryCard::QUEEN_CLUBS => CardNumber::QUEEN_CLUBS,
+            BinaryCard::JACK_CLUBS => CardNumber::JACK_CLUBS,
+            BinaryCard::TEN_CLUBS => CardNumber::TEN_CLUBS,
+            BinaryCard::NINE_CLUBS => CardNumber::NINE_CLUBS,
+            BinaryCard::EIGHT_CLUBS => CardNumber::EIGHT_CLUBS,
+            BinaryCard::SEVEN_CLUBS => CardNumber::SEVEN_CLUBS,
+            BinaryCard::SIX_CLUBS => CardNumber::SIX_CLUBS,
+            BinaryCard::FIVE_CLUBS => CardNumber::FIVE_CLUBS,
+            BinaryCard::FOUR_CLUBS => CardNumber::FOUR_CLUBS,
+            BinaryCard::TREY_CLUBS => CardNumber::TREY_CLUBS,
+            BinaryCard::DEUCE_CLUBS => CardNumber::DEUCE_CLUBS,
+            _ => CardNumber::BLANK,
+        }
+    }
+
     #[must_use]
     fn from_index(index: &str) -> CKCNumber {
         let (rank, suit) = get_rank_and_suit(index);
@@ -918,6 +892,7 @@ mod poker_card_tests {
 
         let card = CardNumber::BLANK as CKCNumber;
         assert_eq!(0b00000000_00000000, card.get_rank_bit());
+        assert_eq!(0b00000000, card.get_rank_prime());
         assert_eq!(CardRank::BLANK, card.get_card_rank());
         assert_eq!(CardSuit::BLANK, card.get_card_suit());
     }
@@ -1014,21 +989,15 @@ mod poker_card_tests {
     fn strip_multiples_flags() {
         assert_eq!(
             CardNumber::ACE_SPADES,
-            CardNumber::ACE_SPADES
-                .flag_as_pair()
-                .strip_multiples_flags()
+            CardNumber::ACE_SPADES.flag_as_pair().strip_multiples_flags()
         );
         assert_eq!(
             CardNumber::ACE_SPADES,
-            CardNumber::ACE_SPADES
-                .flag_as_trips()
-                .strip_multiples_flags()
+            CardNumber::ACE_SPADES.flag_as_trips().strip_multiples_flags()
         );
         assert_eq!(
             CardNumber::ACE_SPADES,
-            CardNumber::ACE_SPADES
-                .flag_as_quads()
-                .strip_multiples_flags()
+            CardNumber::ACE_SPADES.flag_as_quads().strip_multiples_flags()
         );
         assert_eq!(
             CardNumber::ACE_SPADES,
